# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: stats/per_possession_stats.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'stats/per_possession_stats.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n stats/per_possession_stats.proto\x12\tapi.stats\"\xb9\x01\n\rAverageCounts\x12\r\n\x05pass_\x18\x01 \x01(\x02\x12\x0e\n\x06passed\x18\x02 \x01(\x02\x12\x0f\n\x07\x64ribble\x18\x03 \x01(\x02\x12\x1c\n\x14\x64ribble_continuation\x18\x04 \x01(\x02\x12\x0c\n\x04shot\x18\x05 \x01(\x02\x12\x0c\n\x04goal\x18\x06 \x01(\x02\x12\x0e\n\x06\x61ssist\x18\x07 \x01(\x02\x12\x10\n\x08\x61ssisted\x18\x08 \x01(\x02\x12\x0c\n\x04save\x18\t \x01(\x02\x12\x0e\n\x06\x61\x65rial\x18\n \x01(\x02\"\x85\x01\n\x12PerPossessionStats\x12\x30\n\x0e\x61verage_counts\x18\x01 \x01(\x0b\x32\x18.api.stats.AverageCounts\x12\x18\n\x10\x61verage_duration\x18\x02 \x01(\x02\x12\x14\n\x0c\x61verage_hits\x18\x03 \x01(\x02\x12\r\n\x05\x63ount\x18\x04 \x01(\x05')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stats.per_possession_stats_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_AVERAGECOUNTS']._serialized_start=48
  _globals['_AVERAGECOUNTS']._serialized_end=233
  _globals['_PERPOSSESSIONSTATS']._serialized_start=236
  _globals['_PERPOSSESSIONSTATS']._serialized_end=369
# @@protoc_insertion_point(module_scope)



from logging import getLogger
from typing import Dict

import numpy as np
import pandas as pd

from ....analysis.stats.stats import BaseStat
from ....analysis.stats.utils.pandas_utils import sum_deltas_by_truthy_data
from ....generated.api import game_pb2
from ....generated.api.player_pb2 import Player
from ....generated.api.stats.player_stats_pb2 import PlayerStats
from ....json_parser.game import Game


logger = getLogger(__name__)


class ControlsStat(BaseStat):

    def calculate_player_stat(self, player_stat_map: Dict[str, PlayerStats], game: Game, proto_game: game_pb2.Game,
                              player_map: Dict[str, Player], data_frame: pd.DataFrame):
        for player_key, stats in player_map.items():
            try:
                player_name = player_map[player_key].name
                player_data_frame = data_frame[player_name].copy()

                steering_percentage = self.get_analogue_percentage("steer", data_frame, player_name)
                throttle_percentage = self.get_analogue_percentage("throttle", data_frame, player_name)
                is_keyboard = bool((steering_percentage == 0) and (throttle_percentage == 0))

                controller_stats = player_stat_map[player_key].controller
                controller_stats.is_keyboard = is_keyboard
                controller_stats.analogue_steering_input_percent = throttle_percentage
                controller_stats.analogue_throttle_input_percent = steering_percentage
                if 'ball_cam' in player_data_frame:
                    time_ballcam = self.get_ballcam_duration(data_frame, player_data_frame)
                    controller_stats.time_ballcam = time_ballcam
                if 'handbrake' in player_data_frame:
                    time_handbrake = self.get_handbrake_duration(data_frame, player_data_frame)
                    controller_stats.time_handbrake = time_handbrake
            except KeyError as e:
                logger.warning('Player never pressed control %s', e)

    def get_analogue_percentage(self, column: str, data_frame: pd.DataFrame, player_name: str):
        total_frames = len(data_frame[player_name][column])
        count = (data_frame[player_name][column] == 0).sum() + (data_frame[player_name][column] == 128).sum()  + (data_frame[player_name][column] == 255).sum() + data_frame[player_name][column].isna().sum()
        return 100 - ((count * 100) / total_frames)

    @staticmethod
    def get_ballcam_duration(data_frame: pd.DataFrame, player_dataframe: pd.DataFrame) -> np.float64:
        return sum_deltas_by_truthy_data(data_frame, player_dataframe.ball_cam)

    @staticmethod
    def get_handbrake_duration(data_frame: pd.DataFrame, player_dataframe: pd.DataFrame) -> np.float64:
        return sum_deltas_by_truthy_data(data_frame, player_dataframe.handbrake)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: metadata/camera_settings.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'metadata/camera_settings.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1emetadata/camera_settings.proto\x12\x0c\x61pi.metadata\"\xa7\x01\n\x0e\x43\x61meraSettings\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x11\n\tstiffness\x18\x02 \x01(\x02\x12\x0e\n\x06height\x18\x03 \x01(\x02\x12\x18\n\x10transition_speed\x18\x04 \x01(\x02\x12\r\n\x05pitch\x18\x05 \x01(\x02\x12\x14\n\x0cswivel_speed\x18\x06 \x01(\x02\x12\x15\n\rfield_of_view\x18\x07 \x01(\x02\x12\x10\n\x08\x64istance\x18\x08 \x01(\x02')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metadata.camera_settings_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CAMERASETTINGS']._serialized_start=49
  _globals['_CAMERASETTINGS']._serialized_end=216
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: metadata/player_loadout.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'metadata/player_loadout.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dmetadata/player_loadout.proto\x12\x0c\x61pi.metadata\"\xd5\x04\n\rPlayerLoadout\x12\x13\n\x0bload_out_id\x18\x01 \x01(\x05\x12\x0e\n\x06\x62\x61nner\x18\x02 \x01(\x05\x12\r\n\x05\x62oost\x18\x03 \x01(\x05\x12\x0b\n\x03\x63\x61r\x18\x04 \x01(\x05\x12\x16\n\x0egoal_explosion\x18\x05 \x01(\x05\x12\x0c\n\x04skin\x18\x06 \x01(\x05\x12\r\n\x05trail\x18\x07 \x01(\x05\x12\x0e\n\x06wheels\x18\x08 \x01(\x05\x12\x0f\n\x07version\x18\t \x01(\x05\x12\x0e\n\x06topper\x18\n \x01(\x05\x12\x0f\n\x07\x61ntenna\x18\x0b \x01(\x05\x12\x14\n\x0c\x65ngine_audio\x18\x0c \x01(\x05\x12\x15\n\ravatar_border\x18\x16 \x01(\x05\x12\x14\n\x0c\x62\x61nner_paint\x18\r \x01(\x05\x12\x13\n\x0b\x62oost_paint\x18\x0e \x01(\x05\x12\x11\n\tcar_paint\x18\x0f \x01(\x05\x12\x1c\n\x14goal_explosion_paint\x18\x10 \x01(\x05\x12\x12\n\nskin_paint\x18\x11 \x01(\x05\x12\x13\n\x0btrail_paint\x18\x12 \x01(\x05\x12\x14\n\x0cwheels_paint\x18\x13 \x01(\x05\x12\x14\n\x0ctopper_paint\x18\x14 \x01(\x05\x12\x15\n\rantenna_paint\x18\x15 \x01(\x05\x12\x15\n\rprimary_color\x18\x17 \x01(\x05\x12\x14\n\x0c\x61\x63\x63\x65nt_color\x18\x18 \x01(\x05\x12\x16\n\x0eprimary_finish\x18\x19 \x01(\x05\x12\x15\n\raccent_finish\x18\x1a \x01(\x05\x12\x19\n\x11\x62\x61nner_user_color\x18\x1b \x01(\x03\x12 \n\x18\x61vatar_border_user_color\x18\x1c \x01(\x03')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metadata.player_loadout_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_PLAYERLOADOUT']._serialized_start=48
  _globals['_PLAYERLOADOUT']._serialized_end=645
# @@protoc_insertion_point(module_scope)



import logging
from typing import Dict

import numpy as np
import pandas as pd
from ucarball.analysis.constants.field_constants import FieldConstants

from ucarball.analysis.stats.utils.pandas_utils import sum_deltas_by_truthy_data
from ....analysis.stats.stats import BaseStat
from ....generated.api import game_pb2
from ....generated.api.player_pb2 import Player
from ....generated.api.stats.player_stats_pb2 import PlayerStats
from ....json_parser.actor.boost import BOOST_PER_SECOND
from ....json_parser.game import Game

logger = logging.getLogger(__name__)


class DemoStat(BaseStat):
    def calculate_player_stat(self, player_stat_map: Dict[str, PlayerStats], game: Game, proto_game: game_pb2.Game,
                              player_map: Dict[str, Player], data_frame: pd.DataFrame):
        player_demo_counts = {}
        player_got_demoed_counts = {}
        for demo in game.demos:
            attacker = demo['attacker'].online_id
            victim = demo['victim'].online_id
            if attacker not in player_demo_counts:
                player_demo_counts[attacker] = 1
            else:
                player_demo_counts[attacker] += 1
            if victim not in player_got_demoed_counts:
                player_got_demoed_counts[victim] = 1
            else:
                player_got_demoed_counts[victim] += 1
        for player in player_demo_counts:
            player_stat_map[player].demo_stats.num_demos_inflicted = player_demo_counts[player]
        for player in player_got_demoed_counts:
            player_stat_map[player].demo_stats.num_demos_taken = player_got_demoed_counts[player]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: metadata/mutators.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'metadata/mutators.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17metadata/mutators.proto\x12\x0c\x61pi.metadata\"Q\n\x08Mutators\x12)\n\tball_type\x18\x01 \x01(\x0e\x32\x16.api.metadata.BallType\x12\x1a\n\x12game_mutator_index\x18\x02 \x01(\x05*M\n\x08\x42\x61llType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x01\x12\x0e\n\nBASKETBALL\x10\x02\x12\x08\n\x04PUCK\x10\x03\x12\x0c\n\x08\x43UBEBALL\x10\x04\x12\x0c\n\x08\x42REAKOUT\x10\x05')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metadata.mutators_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_BALLTYPE']._serialized_start=124
  _globals['_BALLTYPE']._serialized_end=201
  _globals['_MUTATORS']._serialized_start=41
  _globals['_MUTATORS']._serialized_end=122
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: stats/rumble.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'stats/rumble.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12stats/rumble.proto\x12\tapi.stats\"X\n\x0bRumbleStats\x12\x31\n\x0crumble_items\x18\x01 \x03(\x0b\x32\x1b.api.stats.RumbleItemsUsage\x12\x16\n\x0epre_item_goals\x18\x02 \x01(\x05\"h\n\x10RumbleItemsUsage\x12 \n\x04item\x18\x01 \x01(\x0e\x32\x12.api.stats.PowerUp\x12\x0c\n\x04used\x18\x02 \x01(\x05\x12\x0e\n\x06unused\x18\x03 \x01(\x05\x12\x14\n\x0c\x61verage_hold\x18\x04 \x01(\x02*\xd7\x01\n\x07PowerUp\x12\x0f\n\x0b\x42\x41LL_FREEZE\x10\x01\x12\x17\n\x13\x42\x41LL_GRAPPLING_HOOK\x10\x02\x12\x0e\n\nBALL_LASSO\x10\x03\x12\x0c\n\x08\x42\x41TARANG\x10\x03\x12\x0f\n\x0b\x42\x41LL_SPRING\x10\x04\x12\x0f\n\x0b\x42\x41LL_VELCRO\x10\x05\x12\x12\n\x0e\x42OOST_OVERRIDE\x10\x06\x12\x0e\n\nCAR_SPRING\x10\x07\x12\x10\n\x0cGRAVITY_WELL\x10\x08\x12\x0e\n\nSTRONG_HIT\x10\t\x12\x0b\n\x07SWAPPER\x10\n\x12\x0b\n\x07TORNADO\x10\x0b\x1a\x02\x10\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stats.rumble_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_POWERUP']._loaded_options = None
  _globals['_POWERUP']._serialized_options = b'\020\001'
  _globals['_POWERUP']._serialized_start=230
  _globals['_POWERUP']._serialized_end=445
  _globals['_RUMBLESTATS']._serialized_start=33
  _globals['_RUMBLESTATS']._serialized_end=121
  _globals['_RUMBLEITEMSUSAGE']._serialized_start=123
  _globals['_RUMBLEITEMSUSAGE']._serialized_end=227
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: stats/stats.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'stats/stats.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11stats/stats.proto\x12\tapi.stats\"\x8e\x01\n\nPossession\x12\x17\n\x0fpossession_time\x18\x01 \x01(\x01\x12\x11\n\tturnovers\x18\x02 \x01(\x05\x12\x1c\n\x14turnovers_on_my_half\x18\x03 \x01(\x05\x12\x1f\n\x17turnovers_on_their_half\x18\x04 \x01(\x05\x12\x15\n\rwon_turnovers\x18\x05 \x01(\x05\"\xfe\x02\n\x14PositionalTendencies\x12\x16\n\x0etime_on_ground\x18\x01 \x01(\x02\x12\x17\n\x0ftime_low_in_air\x18\x02 \x01(\x02\x12\x18\n\x10time_high_in_air\x18\x03 \x01(\x02\x12\x1e\n\x16time_in_defending_half\x18\x04 \x01(\x02\x12\x1e\n\x16time_in_attacking_half\x18\x05 \x01(\x02\x12\x1f\n\x17time_in_defending_third\x18\x06 \x01(\x02\x12\x1d\n\x15time_in_neutral_third\x18\x07 \x01(\x02\x12\x1f\n\x17time_in_attacking_third\x18\x08 \x01(\x02\x12\x18\n\x10time_behind_ball\x18\t \x01(\x02\x12\x1a\n\x12time_in_front_ball\x18\n \x01(\x02\x12\x16\n\x0etime_near_wall\x18\x0b \x01(\x02\x12\x16\n\x0etime_in_corner\x18\x0c \x01(\x02\x12\x14\n\x0ctime_on_wall\x18\r \x01(\x02\"e\n\x08\x41verages\x12\x15\n\raverage_speed\x18\x01 \x01(\x02\x12\x1c\n\x14\x61verage_hit_distance\x18\x02 \x01(\x02\x12$\n\x1c\x61verage_distance_from_center\x18\x03 \x01(\x02\"\xd6\x01\n\tHitCounts\x12\x12\n\ntotal_hits\x18\x01 \x01(\x05\x12\x13\n\x0btotal_goals\x18\x02 \x01(\x05\x12\x14\n\x0ctotal_passes\x18\x03 \x01(\x05\x12\x13\n\x0btotal_saves\x18\x04 \x01(\x05\x12\x13\n\x0btotal_shots\x18\x05 \x01(\x05\x12\x16\n\x0etotal_dribbles\x18\x06 \x01(\x05\x12\x1b\n\x13total_dribble_conts\x18\x07 \x01(\x05\x12\x15\n\rtotal_aerials\x18\x08 \x01(\x05\x12\x14\n\x0ctotal_clears\x18\t \x01(\x05\"]\n\x05Speed\x12\x1a\n\x12time_at_slow_speed\x18\x01 \x01(\x02\x12\x1b\n\x13time_at_super_sonic\x18\x02 \x01(\x02\x12\x1b\n\x13time_at_boost_speed\x18\x03 \x01(\x02\"\x85\x02\n\x12\x44\x65tailedCarryStats\x12\x1a\n\x12\x61verage_z_distance\x18\x01 \x01(\x02\x12\x1b\n\x13\x61verage_xy_distance\x18\x02 \x01(\x02\x12\x1f\n\x17\x61verage_ball_z_velocity\x18\x03 \x01(\x02\x12\x1c\n\x14variance_xy_distance\x18\x04 \x01(\x02\x12\x1b\n\x13variance_z_distance\x18\x05 \x01(\x02\x12 \n\x18variance_ball_z_velocity\x18\x06 \x01(\x02\x12\x1b\n\x13\x61verage_carry_speed\x18\x07 \x01(\x02\x12\x1b\n\x13\x64istance_along_path\x18\x08 \x01(\x02')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stats.stats_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_POSSESSION']._serialized_start=33
  _globals['_POSSESSION']._serialized_end=175
  _globals['_POSITIONALTENDENCIES']._serialized_start=178
  _globals['_POSITIONALTENDENCIES']._serialized_end=560
  _globals['_AVERAGES']._serialized_start=562
  _globals['_AVERAGES']._serialized_end=663
  _globals['_HITCOUNTS']._serialized_start=666
  _globals['_HITCOUNTS']._serialized_end=880
  _globals['_SPEED']._serialized_start=882
  _globals['_SPEED']._serialized_end=975
  _globals['_DETAILEDCARRYSTATS']._serialized_start=978
  _globals['_DETAILEDCARRYSTATS']._serialized_end=1239
# @@protoc_insertion_point(module_scope)
